/*
	utility program for the ProWesS loader

	GLOBAL : program to view/set/reset/delete definitions of global variables
	This can only set variables with up to 256 characters.

	written by Joachim Van der Auwera for PROGS
	(c) 1996 by PROGS, Professional & Graphical Software
	started 09 January 1996
*/

#include "str.h"
#include "thing.h"
#include "ProWesS.h"

#define catch(x) if (err=(x)) return err; else

#define GLOBAL_NAME "Global Variables"
#define GLOBAL_GET	0x47455420	/* "GET " */
#define GLOBAL_SET	0x53455420	/* "SET " */
#define GLOBAL_DELE 0x44454c45	/* "DELE" */
#define GLOBAL_FRST 0x46525354	/* "FRST" */
#define GLOBAL_NEXT 0x4e455854	/* "NEXT" */

#define LabelConstant	"constant : "
#define LabelValue		"value : "
#define ItemSet 		"Set constant"
#define ItemDelete		"Delete constant"

#define MAX_NAME	64
#define MAX_VALUE	256

typedef struct {
	PWObject menu;
	PWObject constant, value;
} Global;

/* forward declarations */
Error readall(PWObject object);
Error set(PWObject object);
Error delete(PWObject object);
Error select(PWObject object, char *item);

Error init(char *param)
{
	Global g;
	Error err;
	PWObject win, box;

	catch( PWCreate(NULL, &win, PW_TYPE_OUTLINE,
				PW_OUTLINE_SLEEP,
				PW_OUTLINE_QUIT,
				PW_OUTLINE_QUIT_KEYPRESS, 27,
				PW_OUTLINE_ACTION_WAKE, readall,
				PW_GLOBAL_AUXILIARY, &g,
				NULL) );

	catch( PWCreate(win, &box, PW_TYPE_DIRECTION, NULL) );
	catch( PWCreate(box, NULL, PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ItemSet,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, set,
				NULL) );
	catch( PWCreate(box, NULL, PW_TYPE_LOOSE_ITEM,
				PW_LOOSE_TEXT, ItemDelete,
				PW_LOOSE_CHANGE_STATUS, FALSE,
				PW_LOOSE_ACTION_HIT, delete,
				NULL) );

	catch( PWCreate(win, NULL, PW_TYPE_SEPARATOR, NULL) );
	catch( PWCreate(win, &g.constant, PW_TYPE_EDLINE,
				PW_EDLINE_MAXLENGTH, MAX_NAME,
				NULL) );
	catch( PWCreate(win, &g.value, PW_TYPE_EDLINE,
				PW_EDLINE_MAXLENGTH, MAX_VALUE,
				NULL) );
	catch( PWChange(g.constant,
				PW_EDLINE_EDLINE_AFTER, g.value,
				PW_EDLINE_EDLINE_DOWN, g.value,
				NULL) );
	catch( PWChange(g.value,
				PW_EDLINE_EDLINE_UP, g.constant,
				NULL) );

	catch( PWCreate(win, NULL, PW_TYPE_SEPARATOR, NULL) );
	catch( PWCreate(win, &g.menu, PW_TYPE_MENU,
				PW_MENU_VISIBLE_LINES, 6,
				PW_MENU_SORT_COMPARE, STRCompareCI,
				PW_MENU_ACTION_SELECT, select,
				PW_MENU_NONE_SELECTED,
				NULL) );

	catch( PWCreate(win, NULL, PW_TYPE_LABEL,
				PW_POSITION_LEFT_OF, g.constant,
				PW_LABEL_TEXT, LabelConstant,
				PW_SCALE_FACTOR, 0,
				NULL) );
	catch( PWCreate(win, NULL, PW_TYPE_LABEL,
				PW_POSITION_LEFT_OF, g.value,
				PW_LABEL_TEXT, LabelValue,
				PW_SCALE_FACTOR, 0,
				NULL) );

	readall(win);

	return PWActivate(win);
}

Error readall(PWObject object)
{
	Error err;
	Global *g;
	char name[MAX_NAME], value[MAX_VALUE];

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	PWChange(g->menu,PW_MENU_CLEAR,NULL);

	err=THINGCall(GLOBAL_NAME,GLOBAL_FRST,3,name,value,MAX_VALUE);
	while (!err)
	{
		catch( PWChange(g->menu, PW_MENU_ADD_COPY, name, NULL) );
		err=THINGCall(GLOBAL_NAME,GLOBAL_NEXT,3,name,value,MAX_VALUE);
	}
	return ERR_OK;
}

Error select(PWObject object, char *item)
{
	Error err;
	Global *g;
	char value[MAX_VALUE];

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	err=THINGCall(GLOBAL_NAME,GLOBAL_GET,3,item,value,MAX_VALUE);
	if (err) item="", value[0]='\0';

	PWChange(g->constant, PW_EDLINE_SET, item, NULL);
	PWChange(g->value, PW_EDLINE_SET, value, NULL);

	return ERR_OK;
}

Error set(PWObject object)
{
	Error err;
	Global *g;
	char name[MAX_NAME], value[MAX_VALUE];

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	PWChange(g->constant,PW_EDLINE_GET,MAX_NAME,name,NULL);
	PWChange(g->value,PW_EDLINE_GET,MAX_VALUE,value,NULL);

	THINGCall(GLOBAL_NAME,GLOBAL_SET,2,name,value);

	return readall(object);
}

Error delete(PWObject object)
{
	Error err;
	Global *g;
	char name[MAX_NAME];

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	PWChange(g->constant,PW_EDLINE_GET,MAX_NAME,name,NULL);

	THINGCall(GLOBAL_NAME,GLOBAL_DELE,1,name);

	PWChange(g->constant, PW_EDLINE_SET, "", NULL);
	PWChange(g->value, PW_EDLINE_SET, "", NULL);

	return readall(object);
}
